/*
 * DrvDAC.c
 *
 *  Created on: 2011/11/30
 *      Author: Eric.Lai
 */
#include "DrvDAC.h"
#include "SpecialMacro.h"

/*******************************************************************
*name    : DrvDAC_Open()
*function: enable DAC and setting input IO and set DAO value
*input   : upinput/uninput/uDAO
*output  : 0/others
*Version : v1.0
*date    : 2014-5-30
*modify  : NO
*********************************************************************/
unsigned int DrvDAC_Open(unsigned int uPinput ,unsigned int uNinput,unsigned int uDAO)
{
	unsigned int l_reg,l_data,ReturnVal;
	ReturnVal=0;
	if(uPinput>DAPS_MAX){return E_DRVDAC_ARGUMENT;}
	else{
		l_data=(uPinput<<DAPS)|(DAPS_MAX<<DAPS_MASK);
	}
	if(uNinput>DANS_MAX){return E_DRVDAC_ARGUMENT;}
	else{
		l_data|=((uNinput<<DANS)|(DANS_MAX<<DANS_MASK));
	}
	l_data|=((1<<DAOE)|(1<<DAOE_MASK)|(1<<ENDA)|(1<<ENDA_MASK));
	l_reg=R_DAC_CTRL1;
	outw(l_reg,l_data);
	if(uDAO>DABIT_MAX){return E_DRVDAC_ARGUMENT;}
	else{l_data=uDAO|0XFF00;}
	l_reg=R_DAC_CTRL2;
	outw(l_reg,l_data);
	return ReturnVal;
}

void DrvDAC_Close(void)
{
	unsigned int l_reg,l_data;
	l_reg=R_DAC_CTRL1;
	l_data=(1<<DAOE_MASK)|(1<<(ENDA_MASK));
	outw(l_reg,l_data);
}

void DrvDAC_Enable(void)
{
	unsigned int l_reg,l_data;
	l_reg=R_DAC_CTRL1;
	l_data=(1<<ENDA)|(1<<(ENDA_MASK));
	outw(l_reg,l_data);
}

void DrvDAC_Disable(void)
{
	unsigned int l_reg,l_data;
	l_reg=R_DAC_CTRL1;
	l_data=(1<<(ENDA_MASK));
	outw(l_reg,l_data);
}

void DrvDAC_EnableOutput(void)
{
	unsigned int l_reg,l_data;
	l_reg=R_DAC_CTRL1;
	l_data=(1<<DAOE)|(1<<(DAOE_MASK));
	outw(l_reg,l_data);
}

void DrvDAC_DisableOutput(void)
{
	unsigned int l_reg,l_data;
	l_reg=R_DAC_CTRL1;
	l_data=(1<<(DAOE_MASK));
	outw(l_reg,l_data);
}

/*******************************************************************
*name    : DrvDAC_Pinput(unsigned int uPinput)
*function: setting DAC positive input
*input   : uPinput
*output  : 0/others
*Version : v1.1
*date    : 2014-5-30
*modify  : NO
*********************************************************************/
unsigned int DrvDAC_Pinput(unsigned int uPinput)
{
	unsigned int l_reg,l_data,ReturnVal;
	ReturnVal=0;

	if(uPinput>DAPS_MAX){return E_DRVDAC_ARGUMENT;}
	else{
		l_data=(uPinput<<DAPS)|(DAPS_MAX<<DAPS_MASK);
	}
	l_reg=R_DAC_CTRL1;
	outw(l_reg,l_data);
	return ReturnVal;
}

unsigned int DrvDAC_Ninput(unsigned int uNinput)
{
	unsigned int l_reg,l_data,ReturnVal;
	ReturnVal=0;
	if(uNinput>DANS_MAX){return E_DRVDAC_ARGUMENT;}
	else{l_data=(uNinput<<DANS)|(DANS_MAX<<DANS_MASK);}
	l_reg=R_DAC_CTRL1;
	outw(l_reg,l_data);
	return ReturnVal;
}

/*******************************************************************
*name    : DrvDAC_DABIT(unsigned int uDABIT)
*function: setting DAO value for controlling the output voltage
*input   : uDABIT
*output  : 0/others
*Version : v1.1
*date    : 2014-5-30
*modify  : NO
*********************************************************************/
unsigned int DrvDAC_DABIT(unsigned int uDABIT)
{
	unsigned int l_reg,l_data,ReturnVal;
	ReturnVal=0;
	if(uDABIT>DABIT_MAX){return E_DRVDAC_ARGUMENT;}
	else{l_data=uDABIT|0xff00;}
	l_reg=R_DAC_CTRL2;
	outw(l_reg,l_data);
	return ReturnVal;
}

/*******************************************************************
*name    : unsigned int DrvDAC_SetoutputIO(unsigned int uio)
*function: set PT3.1 as the DAC output IO or not
*input   : uio: 0 disable PT3.1 NOT DAC output mode; 1 enable PT3.1 is DAC output mode
*output  : 0 success; 1 fail
*Version : v1.0
*date    : 2014-5-30
*modify  : no
*********************************************************************/
unsigned int DrvDAC_SetoutputIO(unsigned int uio)
{
	unsigned int l_reg,l_data=0;
	if(uio<2)
	{
		l_reg=PT3_28;
		l_data=0x02000000|(uio<<17);
		outw(l_reg,l_data);
		return E_SUCCESS;
	}
	else return E_FAIL;
}
