/*
 *  DrvLCD.c
 *
 *  Created on: 2014/5/31
 *  Author: MARCK
 */
#include "DrvLCD.h"
#include "SpecialMacro.h"
/**********************************************************************************
*NAME  : DrvLCD_EnableCLK()
*FUNCT : SETTING LCD1 AND LCD2 CLOCK
*INPUT :
*OUTPUT: 0 SUCCESS; 1 FAIL
*VER   : V1.0
*DATE  : 2014-6-4
*MODIFY:
*********************************************************************************/
unsigned char DrvLCD_EnableCLK(unsigned int uLCD1,unsigned int uLCD2,unsigned int usource)
{
	unsigned int l_reg,l_data;
	if((uLCD1>LCDO_MAX)||(uLCD2>LCDE_MAX)||(usource>1))
		return E_FAIL;
	l_data=(uLCD1<<LCDO)|(0X7<<LCDO_MASK)|(uLCD2<<LCDE)|(0X7<<LCDE_MASK)|(usource<<LCKS)|(0X1<<LCKS_MASK);
	l_reg=LCD_CLOCK_CTRL;
	outw(l_reg,l_data);
	return E_SUCCESS;
}
/**********************************************************************************
*NAME  : DrvLCD_DisplayMode()
*FUNCT : SETTING LCD DISPLAY MODE
*INPUT : UDISMODE
*        0 NORMAL ; 1 ALL PIXELS ON; 2 ALL PIXELS OFF; 3 NORMAL MODE
*
*OUTPUT: 0 SUCCESS; 1 FAIL
*VER   : V1.0
*DATE  : 2014-6-4
*MODIFY:
*********************************************************************************/
unsigned char DrvLCD_DisplayMode(unsigned int uDISMODE)
{
	unsigned int l_reg,l_data;
	if(uDISMODE>DSP_MAX)
		return E_FAIL;
	l_data=(uDISMODE<<DSP)|(DSP_MAX<<DSP_MASK);
	l_reg=LCD_CTRL0;
	outw(l_reg,l_data);
	return E_SUCCESS;
}
/**********************************************************************************
*NAME  : DrvLCD_VLCDMode()
*FUNCT : SETTING VLCD MODE
*INPUT : UVLCDMODE
*        0 DISABLE ; 1 R_TYPE; 2  3.3V; 3  3.0V; 4  2.7V; 5  2.4V; 6/7 DISABLE; 
*
*OUTPUT: 0 SUCCESS; 1 FAIL
*VER   : V1.2
*DATE  : 2017-12-11
*MODIFY: 
2: VLCD~3.43V
3: VLCD~3.16V
4: VLCD~2.73V
5: VLCD~2.55V
*********************************************************************************/
unsigned char DrvLCD_VLCDMode(unsigned int uVLCDMODE)
{
	unsigned int l_reg,l_data,l_enable;
	if(uVLCDMODE>VLCD_MAX)
		return E_FAIL;

  l_reg=LCD_CTRL0;

  switch(uVLCDMODE)
  {
  	case 7 :
			outw(l_reg,0x0F07);  //LCD buffer control=off
      break;

	case 6 :
			outw(l_reg,0x0F06);  //LCD buffer control=off
      break;

	case 5 :   //VLCD=2.4V
			//outw(0x41F24,0x0301);
			//outw(0x41B00,0x0705);
		    l_data=inw(VLCD_TRIM)&0X07f;
		    l_enable=0x0f00+((l_data>>3)&0xf);
		    outw(0x41F24,l_enable);
		    l_data=l_data|0x08;  //LCD buffer control=on
		    l_data=(l_data<<VLCD)|(0x0f<<VLCD_MASK);
			outw(l_reg,l_data);
      break;

	case 4 :   //VLCD=2.7V
			//outw(0x41F24,0x0303);
			//outw(0x41B00,0x0704);
			l_data=(inw(0x42740)&0X07f00)>>8;
		    l_enable=0x0f00+((l_data>>3)&0xf);
		    outw(0x41F24,l_enable);
		    l_data=l_data|0x08;  //LCD buffer control=on
		    l_data=(l_data<<VLCD)|(0x0f<<VLCD_MASK);
			outw(l_reg,l_data);
		break;

	case 3 :   //VLCD=3.0V
			//outw(0x41F24,0x0300);
			//outw(0x41B00,0x0704);
			l_data=(inw(0x42740)&0X07f000000)>>24;
		    l_enable=0x0f00+((l_data>>3)&0xf);
		    outw(0x41F24,l_enable);
		    l_data=l_data|0x08;  //LCD buffer control=on
		    l_data=(l_data<<VLCD)|(0x0f<<VLCD_MASK);
			outw(l_reg,l_data);
		break;

	case 2 :   //VLCD=3.3V
			//outw(0x41F24,0x0300);
			//outw(0x41B00,0x0703);
			l_data=(inw(0x42740)&0X07f0000)>>16;
		    l_enable=0x0f00+((l_data>>3)&0xf);
		    outw(0x41F24,l_enable);
		    l_data=l_data|0x08;  //LCD buffer control=on
		    l_data=(l_data<<VLCD)|(0x0f<<VLCD_MASK);
			outw(l_reg,l_data);
		break;

	case 1 :
			outw(l_reg,0x0F09);  //LCD buffer control=on
	  break;

    case 0 :
			outw(l_reg,0x0F00);  //LCD buffer control=off
      break;

	default :
		break;
	}

	return E_SUCCESS;
}
/**********************************************************************************
*NAME  : DrvLCD_LcdDuty()
*FUNCT : SETTING LCD DUTY MODE
*INPUT : UDUTY
*        0 1/3 DUTY ; 1 1/4 DUTY; 2  1/5 DUTY; 3  1/6 DUTY; 
*
*OUTPUT: 0 SUCCESS; 1 FAIL
*VER   : V1.0
*DATE  : 2014-6-4
*MODIFY:
*********************************************************************************/
unsigned char DrvLCD_LcdDuty(unsigned int uDUTY)
{
	unsigned int l_reg,l_data;
	if (uDUTY>DUTY_MAX)
		return E_FAIL;
	l_data=(uDUTY<<DUTY)|(DUTY_MAX<<DUTY_MASK);
	l_reg=LCD_CTRL0;
	outw(l_reg,l_data);
	return E_SUCCESS;
}
/**********************************************************************************
*NAME  : DrvLCD_LCDBuffer()
*FUNCT : SETTING LCD OUTPUT BUFFER BEING ON OR OFF
*INPUT : UBEN
*        0 DISABLE ; 1 ENABLE; 
*
*OUTPUT: 0 SUCCESS; 1 FAIL;
*VER   : V1.0
*DATE  : 2014-6-4
*MODIFY:
*********************************************************************************/
unsigned char DrvLCD_LCDBuffer(unsigned int uBEN)
{
	unsigned int l_reg,l_data;
	if(uBEN>1)
		return E_FAIL;
	l_data=(uBEN<<BEN)|(1<<BEN_MASK);
	l_reg=LCD_CTRL0;
	outw(l_reg,l_data);
	return E_SUCCESS;
}
/**********************************************************************************
*NAME  : DrvLCD_SwpCOMSEG()
*FUNCT : FLIP COM AND SEG ORDER
*INPUT : UFLIP
*        0 NORMAL ; 1 FLIP; 
*OUTPUT: 0 SUCCESS; 1 FAIL;
*VER   : V1.0
*DATE  : 2014-6-4
*MODIFY:
*********************************************************************************/
unsigned char DrvLCD_SwpCOMSEG(unsigned int uflip)
{
	unsigned int l_reg,l_data;
	if(uflip>1)
		return E_FAIL;
	l_data=(uflip<<FLIP)|(1<<FLIP_MASK);
	l_reg=LCD_CTRL0;
	outw(l_reg,l_data);
	return E_SUCCESS;
}
/**********************************************************************************
*NAME  : DrvLCD_IOMode()
*FUNCT : SETTING THE IO port as the LCD MODE OR GPIO MODE;
*INPUT : uport
*        0 PT6LEN ; 1 PT7LEN; 2  PT8LEN; 3 PT9LEN; 4 PT10LEN; 5 COM5/COM4; 
*        uIOMODE :0~0XFF,bit operation
*OUTPUT: 0 SUCCESS; 1 FAIL;
*VER   : V1.0
*DATE  : 2015-6-9
*MODIFY:PT6~PT9д㷨д
*********************************************************************************/
unsigned char DrvLCD_IOMode(unsigned int uport,unsigned int uIOMODE)
{
	unsigned int l_reg,l_data;
	if(uport>5)
		return E_FAIL;
	else if(uport<4)
		{
			l_data=inw(LCD_CTRL1)&(0XFFFFFFFF^(0xff<<(uport*8)));
			l_data=((uIOMODE<<(uport*8))|l_data); //(inw(LCD_CTRL1)&0XFFFFFFFF));
			l_reg=LCD_CTRL1;
		}
	else if(uport==4)
		{
			//l_data=(((uIOMODE&0X3)<<((uport-4)*2))|(inw(LCD_CTRL2)&0XFFFFFFFc));
			l_data=((uIOMODE&0X3)|(inw(LCD_CTRL2)&0XFFFFFFFc));
			l_reg=LCD_CTRL2;
		}
	else
		{
			l_data=(((uIOMODE&0X3)<<((uport-4)*2))|(inw(LCD_CTRL2)&0XFFFFFFF3));
			l_reg=LCD_CTRL2;
		}
	outw(l_reg,l_data);
	return E_SUCCESS;
}
/**********************************************************************************
*NAME  : DrvLCD_WriteData()
*FUNCT : write data to SEG
*INPUT : uSEG  LCD seg LCD0~LCD17
*        PT6:  
*             PT6.0~1 LCD1
*             PT6.2~3 LCD2
*             PT6.4~5 LCD3
*             PT6.6~7 LCD4
*        PT7:  
*             PT7.0~1 LCD5
*             PT7.2~3 LCD6
*             PT7.4~5 LCD7
*             PT7.6~7 LCD8
*        PT8:  
*             PT8.0~1 LCD9
*             PT8.2~3 LCD10
*             PT8.4~5 LCD11
*             PT8.6~7 LCD12
*        PT9:  

*             PT9.0~1 LCD13
*             PT9.2~3 LCD14
*             PT9.4~5 LCD15
*             PT9.6~7 LCD16
*        PT10:  
*             PT10.0~1 LCD17 
*        data : the data write to seg
*OUTPUT: 0 SUCCESS; 1 FAIL;
*VER   : V1.0
*DATE  : 2014-6-4
*MODIFY: 
*2016/03/04  ޸дʾ㷨PT6~10ʹGPIOΪLCDģʽ趨жĳSEGǷ趨ΪGPIOSEGݲд룻
*********************************************************************************/
unsigned char DrvLCD_WriteData(unsigned int uSEG,unsigned int data)
{
	unsigned int l_reg,l_data,l_temp,l_duty;

	if(uSEG>LCD_BUFFER)
		return E_FAIL;
	l_temp=(((inw(LCD_CTRL0)>>DUTY)&0X03)+3);
	l_duty=0x3f>>(6-l_temp);
	if(uSEG==17) //PT10
	{
		l_data=inw(LCD_CTRL2)&0X03;
	}
	else if(uSEG>0)  //about PT6~PT9
	{
		l_data=(inw(LCD_CTRL1)>>((uSEG-1)*2))&0X3;
	}
	else if(uSEG==0)
	{
		l_data=0x3;
	}
    if(l_data==0x3)
    {
    	l_data=0xff00ff00;
    }
    else if(l_data==0x2)
    {
    	l_data=0xff000000;
    }
    else if(l_data==0x01)
    {
    	l_data=0x0000ff00;
    }
    else if(l_data==0)
    	l_data=0x0;
	//l_data=((data&0x03f)<<16)|((data&0xf0)>>l_temp)|0xff00ff00;
	l_data=((data&l_duty)<<16)|((data>>l_temp)&l_duty)|l_data;
	if(uSEG==0)
		l_reg=GPIO_LCD0;
	else
		l_reg=GPIO_LCD1+(uSEG-1)*0x04;
	outw(l_reg,l_data);
	return E_SUCCESS;
}
/**************************************************************
name    : unsigned char DrvLCD_VLCDTrim(short Umode)
function: ͨãûѡͬVLCDѹУ
input:   umode:    1: VLCD~3.43V
                   2: VLCD~3.16V
                   3: VLCD~2.93V
                   4: VLCD~2.73V
                   5: VLCD~2.55V
output :  0 SUCCESS/1 FAIL
note      ʽ
        
date   :   2014-10-17
modify :
      2016-4-19:
      Ĵ    
      2016-4-28:
      HY16F198/HY16F198B   
**************************************************************/
unsigned char DrvLCD_VLCDTrim(short Umode)
{
	unsigned int l_reg,l_enable,l_data;
	unsigned char l_flag;
	if((Umode==0)||(Umode>5))
		return E_FAIL;	
	if((inw(0x40104)&0x00ff0000)==0x00c00000)  
		l_flag=1;
	else if((inw(0x40104)&0x00ff0000)==0x001c0000)
		l_flag=2;
  switch(Umode)
  {
	case 5 :       //else if(Umode==5)
			l_data=inw(VLCD_TRIM)&0X07f;
      break;
      
	case 4 :      //else if(Umode==4){
		if(l_flag==1)
			l_data=(inw(0x41f00)&0X07f00)>>8;		
		else if(l_flag==2) 
			l_data=(inw(0x42740)&0X07f00)>>8;
		break;
		
	case 3 :     //else if(Umode==3){
		if(l_flag==1)
			l_data=inw(0x41f00)&0X07f;		
		else if(l_flag==2)
			l_data=inw(0x42740)&0X07f;
		break;
		
	case 2 :    //else if(Umode==2)
		if(l_flag==1)
			l_data=(inw(0x41f04)&0X07f00)>>8;	
		else if(l_flag==2)
			l_data=(inw(0x42740)&0X07f000000)>>24;	
		break;
		
	case 1 :   //else if(Umode==1)
		if(l_flag==1)
			l_data=inw(0x41f04)&0X07f;	
		else if(l_flag==2)
			l_data=(inw(0x42740)&0X07f0000)>>16;
	  break;
	default :
		break;
	}	  
	l_enable=0x0f00+((l_data>>3)&0xf);
	outw(0x41F24,l_enable);	
	
	l_data=l_data&0x07;
	l_data=(l_data<<VLCD)|(VLCD_MAX<<VLCD_MASK);
	l_reg=LCD_CTRL0;
	outw(l_reg,l_data);

	
	return E_SUCCESS;
}
